#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
# SevenSegmentDisplay Demo vs. 0.1 (2020)
#
# Written by E. A. Tacao <mailto@tacao.com.br>, (C) 2020
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     (1) Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#
#     (2) Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#
#     (3)The name of the author may not be used to
#     endorse or promote products derived from this software without
#     specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#


import wx, time
import wx.lib.buttons as buttons
import colourselect2 as csel

from sevensegment import SevenSegmentDisp

#-------------------------------------------------------------------------------

value = "3.14159265358979323846264338327950288419716939937510"
digits = 5

#-------------------------------------------------------------------------------

class StaticTitleBar(wx.Panel):
    def __init__(self, parent, id=-1, size=None,
                 label="", style=["horizontal"], name="StaticTitleBar"):
        wx.Panel.__init__(self, parent, -1, name=name)

        self.style = style

        if "horizontal" in style:
            boxh = wx.BoxSizer(wx.HORIZONTAL)
            boxv = wx.BoxSizer(wx.VERTICAL)

            if "prepend segment" in style:
                lsize = (7, -1); lborder = 3
            else:
                lsize = (0, 0); lborder = 0
            line = wx.StaticLine(self, -1, size=lsize)
            boxv.Add(line, 1, flag=wx.ALIGN_CENTRE)
            boxh.Add(boxv, flag=wx.ALIGN_CENTRE|wx.RIGHT, border=lborder)

            self.label = wx.StaticText(self, -1, label)
            if label:
                lborder = 3
            else:
                lborder = 0
                self.label.SetSize((0, 0)); self.label.SetMinSize((0, 0))
            boxh.Add(self.label, flag=wx.ALIGN_CENTRE|wx.RIGHT, border=lborder)

            boxv = wx.BoxSizer(wx.VERTICAL)
            line = wx.StaticLine(self, -1)
            line.SetMinSize((7, -1))
            boxv.Add(line, 1, flag=wx.ALIGN_CENTRE|wx.EXPAND)
            boxh.Add(boxv, 1, flag=wx.ALIGN_CENTRE)

        else:
            boxh = wx.BoxSizer(wx.VERTICAL)
            boxv = wx.BoxSizer(wx.HORIZONTAL)

            if "prepend segment" in style:
                lsize = (-1, 7); lborder = 3
            else:
                lsize = (0, 0); lborder = 0
            line = wx.StaticLine(self, -1, size=lsize, style=wx.LI_VERTICAL)
            boxv.Add(line, 1, flag=wx.ALIGN_CENTRE)
            boxh.Add(boxv, flag=wx.ALIGN_CENTRE|wx.RIGHT, border=lborder)

            self.label = wx.StaticText(self, -1, label)
            if label:
                lborder = 3
            else:
                lborder = 0
                self.label.SetSize((0, 0)); self.label.SetMinSize((0, 0))
            boxh.Add(self.label, flag=wx.ALIGN_CENTRE|wx.RIGHT, border=lborder)

            boxv = wx.BoxSizer(wx.HORIZONTAL)
            line = wx.StaticLine(self, -1, style=wx.LI_VERTICAL)
            line.SetMinSize((-1, 7))
            boxv.Add(line, 1, flag=wx.ALIGN_CENTRE|wx.EXPAND)
            boxh.Add(boxv, 1, flag=wx.ALIGN_CENTRE)

        self.SetSizer(boxh)
        if size is not None:
            self.SetSize(size); self.SetMinSize(size)


    def GetValue(self):
        return self.label.GetLabel()


    def SetValue(self, v):
        self.label.SetLabel(v)
        if v: size = (-1, -1)
        else: size = (0, 0)
        self.label.SetSize(size); self.label.SetMinSize(size)


    def GetLabel(self):
        return self.GetValue()


    def SetLabel(self, v):
        self.SetValue(v)


    def GetWindowStyleFlag(self):
        return self.style

#-------------------------------------------------------------------------------

class DemoPanelControl(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)

        choices = ['wx.BORDER_SIMPLE', 'wx.BORDER_DOUBLE', 'wx.BORDER_SUNKEN',
                   'wx.BORDER_RAISED', 'wx.BORDER_STATIC', 'wx.BORDER_NONE']

        self.my_value = value
        self.my_digits = digits
        self.tc = 1
        self.timer = wx.Timer(self)

        if "gtk2" in wx.version():
            spin_size = (55, -1)
        else:
            spin_size = (-1, -1)

        button_size = wx.Button.GetDefaultSize()

        sizer = wx.GridBagSizer(3, 3)
        sizer.SetEmptyCellSize((2, 2))

        p = StaticTitleBar(self, -1, label="Demo options ",
                           style=['horizontal'])
        sizer.Add(p, pos=(0, 0), span=(1, 9),
                  flag=wx.EXPAND|wx.LEFT|wx.TOP|wx.BOTTOM, border=5)

        p = wx.RadioButton(self, -1, "Clock", style=wx.RB_GROUP)
        sizer.Add(p, pos=(1, 1), span=(1, 6), flag=wx.EXPAND|wx.LEFT, border=5)
        p.SetToolTip("Transforms the display in a digital clock")

        p = wx.RadioButton(self, -1, "Custom value:")
        sizer.Add(p, pos=(2, 1), span=(1, 3), flag=wx.EXPAND|wx.LEFT|wx.RIGHT,
                  border=5)
        p.SetValue(True)
        p.SetToolTip("Type a custom value to be shown")

        p = wx.TextCtrl(self, -1, value=self.my_value,
                        style=wx.TE_PROCESS_ENTER)
        sizer.Add(p, pos=(2, 4), span=(1, 3), flag=wx.EXPAND)
        p.SetToolTip("Type a custom value to be shown")
        self.custom_value = p

        p = wx.StaticText(self, -1, label = "Digits:", style=wx.ALIGN_RIGHT)
        sizer.Add(p, pos=(3, 1), span=(1, 3),
                  flag=wx.RIGHT|wx.ALIGN_RIGHT|wx.ALIGN_CENTRE_VERTICAL,
                  border=5)

        p = wx.SpinCtrl(self, -1, size=spin_size, min=1, max=50,
                        value="%s" % self.my_digits)
        sizer.Add(p, pos=(3, 4), span=(1, 3))
        self.digits = p
        p.SetToolTip("Select the number of displays")

        p = StaticTitleBar(self, -1, label="Colours ", style=['horizontal'])
        sizer.Add(p, pos=(5, 0), span=(1, 9),
                  flag=wx.EXPAND|wx.LEFT|wx.TOP|wx.BOTTOM, border=5)

        p = wx.StaticText(self, -1, label = "Background:")
        sizer.Add(p, pos = (6, 1), span=(1, 2),
                  flag=wx.LEFT|wx.ALIGN_CENTRE_VERTICAL, border=5)

        p = csel.ColourSelect(self, -1, "",  wx.Colour(0, 0, 0, 255),
                              size=button_size, name="background")
        sizer.Add(p, pos=(6, 3), span=(1, 2),
                  flag=wx.ALIGN_CENTRE_VERTICAL|wx.LEFT, border=5)
        p.SetToolTip("Changes the background colour")

        p = wx.StaticText(self, -1, label="Segment on:")
        sizer.Add(p, pos = (7, 1), span=(1, 2),
                  flag=wx.LEFT|wx.ALIGN_CENTRE_VERTICAL, border=5)

        p = csel.ColourSelect(self, -1, "", wx.Colour(1, 196, 196, 255),
                              size=button_size, name="segment_on")
        sizer.Add(p, pos=(7, 3), span=(1, 2),
                  flag=wx.ALIGN_CENTRE_VERTICAL|wx.LEFT, border=5)
        p.SetToolTip("Changes the colour of the lit segments")

        p = wx.StaticText(self, -1, label="Segment off:")
        sizer.Add(p, pos = (8, 1), span=(1, 2),
                  flag=wx.LEFT|wx.ALIGN_CENTRE_VERTICAL, border=5)

        p = csel.ColourSelect(self, -1, "", wx.Colour(0, 33, 33, 255),
                              size=button_size, name="segment_off")
        sizer.Add(p, pos=(8, 3), span=(1, 2),
                  flag=wx.ALIGN_CENTRE_VERTICAL|wx.LEFT, border=5)
        p.SetToolTip("Changes the colour of the blank segments")

        sizer.Add((10, 10), pos=(9, 0))

        p = wx.StaticLine(self, style=wx.LI_VERTICAL)
        sizer.Add(p, pos=(0, 9), span=(10, 1),
                  flag=wx.EXPAND|wx.LEFT|wx.RIGHT|wx.BOTTOM, border=5)

        p = StaticTitleBar(self, -1, label="Geometry ", style=['horizontal'])
        sizer.Add(p, pos=(0, 10), span=(1, 17),
                  flag=wx.EXPAND|wx.RIGHT|wx.TOP|wx.BOTTOM, border=5)

        p = wx.StaticText(self, -1, label = "Width:", style=wx.ALIGN_RIGHT)
        sizer.Add(p, pos=(1, 11), span=(1, 3),
                  flag=wx.RIGHT|wx.ALIGN_RIGHT|wx.ALIGN_CENTRE_VERTICAL,
                  border=5)

        p = wx.SpinCtrl(self, -1, size=spin_size, min=1, max=500, value="38")
        sizer.Add(p, pos=(1, 14), span=(1, 3), flag=wx.ALIGN_LEFT)
        self.width = p
        p.SetToolTip("Width of the a, d, g segments")

        p = wx.StaticText(self, -1, label = "Height:", style=wx.ALIGN_RIGHT)
        sizer.Add(p, pos=(1, 19), span=(1, 3),
                  flag=wx.RIGHT|wx.ALIGN_RIGHT|wx.ALIGN_CENTRE_VERTICAL,
                  border=5)

        p = wx.SpinCtrl(self, -1, size=spin_size, min=1, max=500, value="38")
        sizer.Add(p, pos=(1, 23), span=(1, 3), flag=wx.ALIGN_LEFT|wx.RIGHT,
                  border=20)
        self.height = p
        p.SetToolTip("Height of the b, c, e, f segments")

        p = wx.StaticText(self, -1, label = "Thickness:", style=wx.ALIGN_RIGHT)
        sizer.Add(p, pos=(2, 11), span=(1, 3),
                  flag=wx.RIGHT|wx.ALIGN_RIGHT|wx.ALIGN_CENTRE_VERTICAL,
                  border=5)

        p = wx.SpinCtrl(self, -1, size=spin_size, min=1, max=500, value="10")
        sizer.Add(p, pos=(2, 14), span=(1, 3), flag=wx.ALIGN_LEFT)
        self.thickness = p
        p.SetToolTip("Changes the segments thickness")

        p = wx.StaticText(self, -1, label = "Separation:", style=wx.ALIGN_RIGHT)
        sizer.Add(p, pos=(2, 19), span=(1, 3),
                  flag=wx.RIGHT|wx.ALIGN_RIGHT|wx.ALIGN_CENTRE_VERTICAL,
                  border=5)

        p = wx.SpinCtrl(self, -1, size=spin_size, min=1, max=500, value="3")
        sizer.Add(p, pos=(2, 23), span=(1, 3), flag=wx.ALIGN_LEFT|wx.RIGHT,
                  border=20)
        p.SetToolTip("Sets spacing between segments")
        self.separation = p

        p = wx.StaticText(self, -1, label = "Tilt:", style=wx.ALIGN_RIGHT)
        sizer.Add(p, pos=(3, 12),
                  flag=wx.RIGHT|wx.ALIGN_CENTRE_VERTICAL|wx.ALIGN_RIGHT,
                  border=5)

        p = wx.Slider(self, -1, value=10, minValue=0, maxValue=28,
                      style=wx.SL_HORIZONTAL|wx.SL_AUTOTICKS)
        sizer.Add(p, pos=(3, 13), span=(1, 10),
                  flag=wx.LEFT|wx.ALIGN_CENTRE_VERTICAL|wx.EXPAND, border=5)
        p.SetToolTip("Tilt display by n degrees")

        p = wx.StaticText(self, -1, label = "10°")
        sizer.Add(p, pos=(3, 23), flag=wx.ALIGN_CENTRE_VERTICAL|wx.ALIGN_LEFT)
        self.tilt = p

        p = StaticTitleBar(self, -1, label="Other ", style=['horizontal'])
        sizer.Add(p, pos=(5, 10), span=(1, 17),
                  flag=wx.EXPAND|wx.RIGHT|wx.TOP|wx.BOTTOM, border=5)

        p = wx.CheckBox(self, -1, "Enable Dot")
        sizer.Add(p, pos=(6, 11), span=(1, 7), flag=wx.EXPAND)
        p.SetValue(True)
        p.SetToolTip("Enables dots")

        p = wx.CheckBox(self, -1, "Enable Colon")
        sizer.Add(p, pos=(6, 19), span=(1, 7), flag=wx.EXPAND)
        p.SetValue(True)
        p.SetToolTip("Enables colons")

        sizer.AddGrowableCol(8)
        sizer.AddGrowableCol(26)
        sizer.AddGrowableRow(9)
        self.SetSizerAndFit(sizer)

        self.disp = DemoDisplay()
        self.disp.Show()
        self.disp.Assemble(self.my_digits)
        self.disp.SetValue(self.my_value)

        self.Bind(wx.EVT_TIMER, self.OnTimer)
        self.Bind(wx.EVT_RADIOBUTTON, self.OnRadio)
        self.Bind(wx.EVT_TEXT, self.OnText)
        self.Bind(wx.EVT_SPINCTRL, self.OnSpin)
        self.Bind(wx.EVT_SLIDER, self.OnSlider)
        self.Bind(csel.EVT_COLOURSELECT, self.OnSelectColour)
        self.Bind(wx.EVT_CHECKBOX, self.OnCheckBox)


    def OnTimer(self, evt):
        self.tc = -self.tc
        t = time.localtime(time.time())
        st = time.strftime("%H%M%S", t)

        vals = [i for i in st]
        if self.tc > 0:
            vals[1] += ":"
            vals[3] += ":"
        val = "".join(vals)

        self.disp.SetValue(val)


    def OnRadio(self, evt):
        label = evt.GetEventObject().GetLabel()
        if label == "Clock":
            self.custom_value.ChangeValue("")
            self.custom_value.Disable()
            self.digits.SetValue(6)
            self.disp.Assemble(6)
            self.disp.SetValue("-"*6)
            self.digits.Disable()
            self.timer.Start(500)

        elif label == "Custom value:":
            self.timer.Stop()
            self.custom_value.Enable()
            self.custom_value.ChangeValue(self.my_value)
            self.digits.Enable()
            self.digits.SetValue(self.my_digits)
            self.disp.Assemble(self.my_digits)
            self.disp.SetValue(self.my_value)


    def OnText(self, evt):
        if evt.GetEventObject() == self.custom_value:
            self.my_value = self.custom_value.GetValue()
            self.disp.SetValue(self.my_value)


    def OnSpin(self, evt):
        obj = evt.GetEventObject()
        val = obj.GetValue()

        if obj == self.digits:
            self.disp.Assemble(val)
            self.disp.SetValue(self.my_value)
            self.my_digits = val

        elif obj == self.thickness:
            self.disp.SetGeometry(thickness=val)

        elif obj == self.width:
            self.disp.SetGeometry(width=val)

        elif obj == self.height:
            self.disp.SetGeometry(height=val)

        elif obj == self.separation:
            self.disp.SetGeometry(separation=val)


    def OnSlider(self, evt):
        val = evt.GetInt()
        self.tilt.SetLabel("%s°" % str(val))
        self.disp.SetTilt(val)


    def OnSelectColour(self, evt):
        obj = evt.GetEventObject()
        val = obj.GetValue()
        name = obj.GetName()
        self.disp.SetColours(**{name: val})


    def OnCheckBox(self, evt):
        obj = evt.GetEventObject()
        val = obj.GetValue()
        label = obj.GetLabel()

        if label == "Enable Dot":
            self.disp.EnableDot(val)
        elif label == "Enable Colon":
            self.disp.EnableColon(val)

#-------------------------------------------------------------------------------

class DemoDisplay(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, -1, "SevenSegmentDisp Demo")

        self.SetSize((500, 180))
        self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.SetSizer(self.sizer)
        self.disps = []
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        self.CentreOnScreen()


    def Assemble(self, num):
        dif = num-len(self.disps)
        if dif > 0:
            # Add displays
            for i in range(0, dif):
                d = SevenSegmentDisp(self)

                if len(self.disps):
                    # Make sure new displays will be set the same way as
                    # the existing ones.
                    d.SetTilt(self.disps[0].GetTilt())
                    d.SetColours(**self.disps[0].GetColours())
                    d.SetGeometry(**self.disps[0].GetGeometry())
                    d.EnableDot(self.disps[0].IsDotEnabled())
                    d.EnableColon(self.disps[0].IsColonEnabled())

                self.sizer.Add(d, 1, flag=wx.EXPAND)
                self.disps.append(d)
        elif dif < 0:
            # Remove displays: detach, destroy, delete.
            for i in range(dif, 0):
                d = self.disps[-1]
                self.sizer.Detach(d)
                d.Destroy()
                self.disps.remove(d)
        self.sizer.Layout()


    def SetTilt(self, val):
        for d in self.disps:
            d.SetTilt(val)


    def SetColours(self, **kwargs):
        for d in self.disps:
            d.SetColours(**kwargs)


    def SetGeometry(self, **kwargs):
        for d in self.disps:
            d.SetGeometry(**kwargs)


    def EnableDot(self, val):
        for d in self.disps:
            d.EnableDot(val)


    def EnableColon(self, val):
        for d in self.disps:
            d.EnableColon(val)


    def SetValue(self, value):
        # This shows how to parse a string to feed individual values
        # to each of the displays.

        cvs = []
        i = 0
        while i < len(value):
            c0 = value[i]
            if i+1 < len(value):
                c1 = value[i+1]
            else:
                c1 = ""
            if i+2 < len(value):
                c2 = value[i+2]
            else: c2 = ""

            if c0 in [".", ":"] and c1 in [".", ":"] and c0 != c1:
                p = " " + c0 + c1
                i += 1

            elif c0 in [".", ":"]:
                p = " " + c0

            else:
                if c1 in [".", ":"] and c2 in [".", ":"] and c1 != c2:
                    p = c0 + c1 + c2
                    i += 2
                elif c1 in [".", ":"] and c2 in [".", ":"] and c1 == c2:
                    p = c0 + c1
                    i += 1
                elif c1 in [".", ":"]:
                    p = c0 + c1
                    i += 1
                else:
                    p = c0

            cvs.append(p)
            i += 1

        # This shows how to right justify the display or truncate the value
        # shown when using multiple displays.
        num = len(self.disps)
        if len(cvs) > num:
            cvs = cvs[:num]
        else:
            cvs = [" "] * (num - len(cvs)) + cvs

        for i in range(0, len(self.disps)):
            self.disps[i].SetValue(cvs[i])


    def OnClose(self, evt):
        wx.GetApp().GetTopWindow().Close()
        evt.Skip()

#-------------------------------------------------------------------------------

class DemoControl(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, -1, "SevenSegmentDisp Demo Control")

        self.p = DemoPanelControl(self)
        self.SetClientSize(self.p.GetSize())
        self.Bind(wx.EVT_CLOSE, self.OnClose)


    def OnClose(self, evt):
        self.p.disp.Destroy()
        evt.Skip()

#-------------------------------------------------------------------------------

class demoApp(wx.App):
    def OnInit(self):
        p = DemoControl()
        self.SetTopWindow(p)
        p.Show()
        return True

#-------------------------------------------------------------------------------

if __name__ == '__main__':
    app = demoApp(0)
    app.MainLoop()


#
##
### eof
